from __future__ import annotations

from typing import Mapping, NewType

from typing_extensions import Unpack

from haitch._attrs import AttributeValue, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

FigcaptionElement = NewType("FigcaptionElement", Element)
"""A `<figcaption>` element."""


def figcaption(
    *children: Child,
    extra_attrs: Mapping[str, AttributeValue] = {},
    **attrs: Unpack[GlobalAttrs],
) -> FigcaptionElement:
    """Represents a caption or legend describing the contents of its parent <figure>.

    Nesting this in a figure element improves accessibility.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/figcaption>
    """
    el = Element("figcaption")(**attrs, **extra_attrs)(*children)
    return FigcaptionElement(el)
